//
//  LionelTokenStream.h
//  Lionel
//
//  Created by Devin Coughlin on 4/8/06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>

@class LionelLexer;
@class LionelToken;

@interface LionelTokenStream : NSObject {
	LionelLexer *_lexer;
	
	NSMutableArray *_tokens;
	
	NSMutableArray *_tokenStagingArea;
	
	NSConditionLock *_tokenStreamConditionLock;
	
	
	BOOL _lexingStarted;
	BOOL _lexingCompleted;
	
	BOOL _ignoreWhitespace;
	BOOL _ignoreComments;
	
	
	/*  The following are used to tell the condition lock what conditions
		imply the availability of a token in a given context:
		
		consumeToken: - expects at least 1 non-ignored token to be available in the token buffer
		lookAhead: (int) n - expects at least non-ignored token to available in the token buffer
		
		scanUntilTokenOfType: - expects at least 1 token of the specified type to be in the token buffer
		scanUntilTokenOfType:value: - expects at least 1 token with the specified type and value to be in the token buffer
	*/
	 	
	int _tokenAvailabilityThreshold;
	
	NSString *_tokenAvailabilityType;
	NSString *_tokenAvailabilityValue;
}

/* Creation */

+ (LionelTokenStream *) tokenStreamWithInputStream: (NSInputStream *) inputStream encoding: (NSStringEncoding) encoding;
+ (LionelTokenStream *) tokenStreamWithString: (NSString *) string;

+ (LionelTokenStream *) tokenStreamWithContentsOfFile: (NSString *) filePath encoding: (NSStringEncoding) encoding;

/* Attributes */

- (BOOL) ignoreWhitespace;
- (void) setIgnoreWhitespace: (BOOL) ignoreWhitespace;

- (BOOL) ignoreComments;
- (void) setIgnoreComments: (BOOL) ignoreComments;

- (LionelLexer *) lexer;

/* Methods */

/* These will block if the requested token is not available */
- (LionelToken *) consumeToken;
- (LionelToken *) lookAhead: (int) lookAheadCount;


/*These methods are useful for error recovery. They will remove all the tokens
  up to and including the specified token. If no such token is found, all tokens
  are removed from the stream.
*/
- (void) scanThroughTokenOfType: (NSString *) tokenType;
- (void) scanThroughTokenOfType: (NSString *) tokenType value: (NSString *) value;
@end
